package adminuser

import (
	"github.com/gin-gonic/gin"
	"net/http"
	"sghgogs.com/micro/common"
	res "sghgogs.com/micro/shopping-service-client/response"
	"sghgogs.com/micro/shopping-service-client/utils"
	pb "sghgogs.com/micro/shopping-service-service/proto"
)

type CreateAdminUserRequest struct {
	Username    string  `json:"username" binding:"required,max=15,min=5"` // 用户名
	Password    string  `json:"password" binding:"required,max=18,min=6"` // 密码
	PhoneNumber string  `json:"phone_number" binding:"required"`          // 手机号
	Email       string  `json:"email"`                                    // 邮箱
	Avatar      string  `json:"avatar"`                                   // 头像
	Roles       []int64 `json:"roles"`                                    // 角色
}

// CreateAdminUser
// @summary 新增管理员
// @Tags AdminUsers
// @Accept json
// @Produce json
// @Param Authorization header string true "Bearer 用户令牌"
// @Param body body CreateAdminUserRequest true "请求body"
// @Success 200 {object} response.ApiResponse "成功"
// @Failure 400 {object} response.ApiResponse "请求错误"
// @Failure 500 {object} response.ApiResponse "内部错误"
// @Router /admin/v1/api/manager/user [post]
func (svc *AdminUser) CreateAdminUser(c *gin.Context) {
	var req CreateAdminUserRequest
	if err := c.ShouldBindJSON(&req); err != nil {
		c.JSON(http.StatusBadRequest, res.ErrorResponse(http.StatusBadRequest, err.Error()))
		return
	}
	if !common.ValidateUsername(req.Username) {
		c.JSON(http.StatusBadRequest, res.ErrorResponse(http.StatusBadRequest, "4到16位（字母，数字，下划线，减号）"))
		return
	}
	if common.IsValidPhoneNumber(req.PhoneNumber) {
		c.JSON(http.StatusBadRequest, res.ErrorResponse(http.StatusBadRequest, "手机号格式有误"))
		return
	}
	if len(req.Email) > 0 && !common.IsValidEmail(req.Email) {
		c.JSON(http.StatusBadRequest, res.ErrorResponse(http.StatusBadRequest, "邮箱格式不对"))
		return
	}

	ctx, _ := utils.CreateContextWithToken(c, svc.Namespace, "ShoppingAdminUserService.CreateAdminUser")
	_, err := svc.Service.CreateAdminUser(ctx, &pb.CreateAdminUserRequest{
		Username:    req.Username,
		Password:    req.Password,
		PhoneNumber: req.PhoneNumber,
		Email:       req.Email,
		Avatar:      req.Avatar,
		Roles:       req.Roles,
	})
	if err != nil {
		code, mgs := res.MicroErrorRequest(err)
		c.JSON(code, res.ErrorResponse(code, mgs))
		return
	}
	c.JSON(http.StatusOK, res.SuccessResponse(nil))
}
