package category

import (
	"fmt"
	"github.com/gin-gonic/gin"
	"net/http"
	model_res "sghgogs.com/micro/shopping-service-client/domain/model/response"
	res "sghgogs.com/micro/shopping-service-client/response"
	"sghgogs.com/micro/shopping-service-client/utils"
	pb "sghgogs.com/micro/shopping-service-service/proto"
	"strconv"
)

type ListResponse struct {
	Items      []model_res.Category `json:"items"`
	TotalCount int64                `json:"total_count"`
}

// GetCategoryList
// @summary 列表
// @Tags 分类管理
// @Accept json
// @Produce json
// @Param Authorization header string true "Bearer 用户令牌"
// @Param page query int true "页码"
// @Param page_size query int true "每页数量"
// @Param keyword query string false "关键字"
// @Param status query string false "状态" Enums(enabled, disabled)
// @Success 200 {object} response.ApiResponse{data=ListResponse} "成功"
// @Failure 400 {object} response.ApiResponse "请求错误"
// @Failure 500 {object} response.ApiResponse "内部错误"
// @Router /admin/v1/api/categorizes [get]
func (svc *Category) GetCategoryList(c *gin.Context) {
	keyword := c.Query("keyword")
	queryPage := c.DefaultQuery("page", "1")
	queryPageSize := c.DefaultQuery("page_size", "10")
	page, _ := strconv.Atoi(queryPage)
	pageSize, _ := strconv.Atoi(queryPageSize)
	ctx, _ := utils.CreateContextWithToken(c, svc.Namespace, "ShoppingCategoryService.GetCategoryList")
	list, err := svc.Service.GetCategoryList(ctx, &pb.GetCategoryListRequest{
		Page:     int32(page),
		PageSize: int32(pageSize),
		Keyword:  keyword,
	})
	if err != nil {
		code, mgs := res.MicroErrorRequest(err)
		c.JSON(code, res.ErrorResponse(code, mgs))
		return
	}
	c.JSON(http.StatusOK, res.SuccessResponse(ListResponse{
		TotalCount: list.TotalCount,
		Items:      svc.getCategorizes(list.Items),
	}))
}

type ListAllResponse struct {
	Items []model_res.Category `json:"items"`
}

// GetAllCategories
// @summary 所有
// @Tags 分类管理
// @Accept json
// @Produce json
// @Param Authorization header string true "Bearer 用户令牌"
// @Param page query int true "页码"
// @Param page_size query int true "每页数量"
// @Param keyword query string false "关键字"
// @Param status query string false "状态" Enums(enabled, disabled)
// @Success 200 {object} response.ApiResponse{data=ListAllResponse} "成功"
// @Failure 400 {object} response.ApiResponse "请求错误"
// @Failure 500 {object} response.ApiResponse "内部错误"
// @Router /admin/v1/api/all_categorizes [get]
func (svc *Category) GetAllCategories(c *gin.Context) {
	ctx, _ := utils.CreateContextWithToken(c, svc.Namespace, "ShoppingCategoryService.GetAllCategories")
	list, err := svc.Service.GetAllCategories(ctx, &pb.GetAllCategoriesRequest{})
	if err != nil {
		code, mgs := res.MicroErrorRequest(err)
		c.JSON(code, res.ErrorResponse(code, mgs))
		return
	}
	c.JSON(http.StatusOK, res.SuccessResponse(ListAllResponse{
		Items: svc.getCategorizes(list.Items),
	}))
}

func (svc *Category) getCategorizes(categorizes []*pb.Category) []model_res.Category {
	items := make([]model_res.Category, 0)
	for _, item := range categorizes {
		category := model_res.Category{
			ID:               item.Id,
			Name:             item.Name,
			ParentCategoryID: item.ParentCategoryId,
			Level:            item.Level,
		}
		if len(item.ImageUrl) > 0 {
			category.ImageURL = fmt.Sprintf("http://192.168.124.2:9000/test%v", item.ImageUrl)
		}
		items = append(items, category)
	}
	return items
}
