package api

import (
	"sghgogs.com/micro/shopping-service-client/domain/model/base"
	model_res "sghgogs.com/micro/shopping-service-client/domain/model/response"
	"sghgogs.com/micro/shopping-service-client/utils"
	pb "sghgogs.com/micro/shopping-service-service/proto"
	"time"
)

func GetProductsRes(products []*pb.Product) []model_res.Product {
	items := make([]model_res.Product, 0)
	for _, item := range products {
		items = append(items, GetProductResItem(item))
	}
	return items
}
func GetProductResItem(item *pb.Product) model_res.Product {
	var unix time.Time
	if item.UpdatedAt > 0 {
		unix = time.Unix(item.UpdatedAt, 0)
	}
	product := model_res.Product{
		ID:            item.Id,
		Name:          item.Name,
		Description:   item.Description,
		Price:         item.Price,
		StockQuantity: item.StockQuantity,
		CreatedAt:     utils.ConvertInt64ToTime(item.CreatedAt),
		UpdatedAt:     &unix,
		CategoryID:    item.CategoryId,
		ImageURL:      item.ImageUrl,
	}
	if item.Category != nil {
		product.Category = GetCategory(item.Category)
	}
	return product
}

func GetCategory(category *pb.Category) *model_res.CategoryLabels {
	return &model_res.CategoryLabels{
		ID:   category.Id,
		Name: category.Name,
	}
}

func GetRoleRes(role *pb.Role) model_res.Role {
	toStatus, _ := base.MapEnumToStatus(role.Status)
	var unix time.Time
	if role.UpdatedAt > 0 {
		unix = time.Unix(role.UpdatedAt, 0)
	}
	return model_res.Role{
		ID:          role.Id,
		Name:        role.Name,
		Description: role.Description,
		CreatedBy:   role.CreatedBy,
		CreatedAt:   utils.ConvertInt64ToTime(role.CreatedAt),
		UpdatedAt:   &unix,
		UpdatedBy:   role.UpdatedBy,
		Status:      toStatus,
		IsReserved:  role.IsReserved,
	}
}

func GetUserRes(user *pb.User) model_res.User {
	toStatus, _ := base.MapEnumToStatus(user.Status)
	var at time.Time
	if user.UpdatedAt > 0 {
		at = utils.ConvertInt64ToTime(user.UpdatedAt)
	}
	return model_res.User{
		ID:          user.Id,
		Username:    user.Username,
		PhoneNumber: user.PhoneNumber,
		Email:       user.Email,
		Avatar:      user.Avatar,
		CreatedAt:   utils.ConvertInt64ToTime(user.CreatedAt),
		UpdatedAt:   &at,
		Status:      toStatus,
	}
}

func GetUsersRes(users []*pb.User) []model_res.User {
	items := make([]model_res.User, 0)
	for _, item := range users {
		items = append(items, GetUserRes(item))
	}
	return items
}

func GetBaseRes(base2 []*pb.Base) []base.ListMapItem {
	items := make([]base.ListMapItem, 0)
	for _, item := range base2 {
		items = append(items, base.ListMapItem{Key: item.Key, Value: item.Value})
	}
	return items
}

func GetAddressResItem(address *pb.Address) model_res.Address {
	return model_res.Address{
		ID:          address.Id,
		UserID:      address.UserId,
		CountryCode: address.CountryCode,
		ProvinceID:  address.ProvinceId,
		CityID:      address.CityId,
		DistrictID:  address.DistrictId,
		Remark:      address.Remark,
		IsDefault:   address.IsDefault,
	}
}

func GetResShoppingCartItems(items []*pb.ShoppingCartItem) []model_res.ShoppingCartItem {
	cartItems := make([]model_res.ShoppingCartItem, 0)
	for _, item := range items {
		cartItems = append(cartItems, GetResShoppingCartItem(item))
	}
	return cartItems
}
func GetResShoppingCartItem(item *pb.ShoppingCartItem) model_res.ShoppingCartItem {
	product := item.Product
	return model_res.ShoppingCartItem{
		ID:             item.Id,
		ShoppingCartID: item.ShoppingCartId,
		ProductID:      item.ProductId,
		Product: model_res.Product{
			ID:            product.Id,
			Name:          product.Name,
			Description:   product.Description,
			Price:         product.Price,
			StockQuantity: product.StockQuantity,
			ImageURL:      product.ImageUrl,
		},
		Quantity:   item.Quantity,
		TotalPrice: item.TotalPrice,
	}
}

type OrderItem struct {
	ID            int64                 `json:"id"`
	UserID        int64                 `json:"user_id"`
	Username      string                `json:"username"` // 用户名
	Address       model_res.Address     `json:"address"`  // 地址
	OrderItems    []model_res.OrderItem `json:"order_items"`
	TotalAmount   int64                 `json:"total_amount"`   // 订单金额
	Status        int                   `json:"status"`         // 订单状态
	PaymentMethod int                   `json:"payment_method"` // 订单支付方式
	ApproveName   string                `json:"approve_name"`   // 操作人
	CreatedAt     time.Time             `json:"created_at"`
	UpdatedAt     *time.Time            `json:"updated_at"`
}

func GetOrderListRes(orders []*pb.Order) []OrderItem {
	items := make([]OrderItem, 0)
	for _, item := range orders {
		items = append(items, GetOrderRes(*item))
	}
	return items
}

func GetOrderRes(order pb.Order) OrderItem {
	item := OrderItem{
		ID:            order.Id,
		UserID:        order.UserId,
		Username:      order.User.Username,
		TotalAmount:   order.TotalAmount,
		Status:        int(order.Status),
		PaymentMethod: int(order.PaymentMethod),
		ApproveName:   order.Approver.Username,
		CreatedAt:     utils.ConvertInt64ToTime(order.CreatedAt),
	}
	if order.UpdatedAt > 0 {
		unix := time.Unix(order.UpdatedAt, 0)
		item.UpdatedAt = &unix
	}
	return item
}

func GetOrderItemsRes(items []*pb.OrderItem) []model_res.OrderItem {
	orderItems := make([]model_res.OrderItem, 0)
	for _, item := range items {
		orderItems = append(orderItems, GetOrderItemRes(item))
	}
	return orderItems
}

func GetOrderItemRes(item *pb.OrderItem) model_res.OrderItem {
	orderItem := model_res.OrderItem{
		ID:        item.Id,
		OrderID:   item.OrderId,
		ProductID: item.ProductId,
		Quantity:  item.Quantity,
		Subtotal:  item.Subtotal,
	}
	if item.Product != nil {
		orderItem.Product = GetProductResItem(item.Product)
	}
	return orderItem
}

// Payment 支付

func GetPaymentResItem(payment *pb.Payment) model_res.Payment {
	method, _ := base.MapEnumToPaymentMethod(payment.PaymentMethod)
	status, _ := base.MapEnumToPaymentStatus(payment.Status)
	return model_res.Payment{
		ID:            payment.Id,
		OrderID:       payment.OrderId,
		Amount:        payment.Amount,
		Status:        status,
		PaymentMethod: method,
		CreatedAt:     utils.ConvertInt64ToTime(payment.CreatedAt),
		UpdatedAt:     nil,
	}
}
