package permission

import (
	"github.com/gin-gonic/gin"
	"net/http"
	"sghgogs.com/micro/shopping-service-client/api"
	model_res "sghgogs.com/micro/shopping-service-client/domain/model/response"
	res "sghgogs.com/micro/shopping-service-client/response"
	"sghgogs.com/micro/shopping-service-client/utils"
	pb "sghgogs.com/micro/shopping-service-service/proto"
	"strconv"
)

type ListResponse struct {
	Items      []model_res.Permission `json:"items"`
	TotalCount int64                  `json:"total_count"`
}

// GetPermissionList
// @summary 列表
// @Tags 权限项管理
// @Accept json
// @Produce json
// @Param Authorization header string true "Bearer 用户令牌"
// @Param page query int true "页码"
// @Param page_size query int true "每页数量"
// @Param keyword query string false "关键字"
// @Param status query string false "状态" Enums(enabled, disabled)
// @Success 200 {object} response.ApiResponse{data=ListResponse} "成功"
// @Failure 400 {object} response.ApiResponse "请求错误"
// @Failure 500 {object} response.ApiResponse "内部错误"
// @Router /admin/v1/api/permissions [get]
func (svc *Permission) GetPermissionList(c *gin.Context) {
	keyword := c.Query("keyword")
	queryPage := c.DefaultQuery("page", "1")
	queryPageSize := c.DefaultQuery("page_size", "10")
	page, _ := strconv.Atoi(queryPage)
	pageSize, _ := strconv.Atoi(queryPageSize)

	ctx, _ := utils.CreateContextWithToken(c, svc.Namespace, "ShoppingPermissionService.GetPermissionList")
	list, err := svc.Service.GetPermissionList(ctx, &pb.GetPermissionListRequest{
		Page:     int32(page),
		PageSize: int32(pageSize),
		Keyword:  keyword,
	})
	if err != nil {
		code, mgs := res.MicroErrorRequest(err)
		c.JSON(code, res.ErrorResponse(code, mgs))
		return
	}
	c.JSON(http.StatusOK, res.SuccessResponse(ListResponse{
		TotalCount: list.TotalCount,
		Items:      svc.getPermissionsRes(list.Items),
	}))
}

// GetPermission
// @summary 详情
// @Tags 权限项管理
// @Accept json
// @Produce json
// @Param Authorization header string true "Bearer 用户令牌"
// @Param    id   path      int  true  "权限项 ID"
// @Success 200 {object} response.ApiResponse{data=model_res.Permission} "成功"
// @Failure 400 {object} response.ApiResponse "请求错误"
// @Failure 500 {object} response.ApiResponse "内部错误"
// @Router /admin/v1/api/permission/{permission_id} [get]
func (svc *Permission) GetPermission(c *gin.Context) {
	permissionID, err := strconv.ParseInt(c.Param("permission_id"), 10, 64)
	if err != nil || permissionID <= 0 {
		c.JSON(http.StatusBadRequest, gin.H{"error": "Invalid permissionID"})
		return
	}
	ctx, _ := utils.CreateContextWithToken(c, svc.Namespace, "ShoppingPermissionService.GetPermission")
	result, err := svc.Service.GetPermission(ctx, &pb.GetPermissionRequest{PermissionId: permissionID})
	if err != nil {
		code, mgs := res.MicroErrorRequest(err)
		c.JSON(http.StatusBadRequest, res.ErrorResponse(code, mgs))
		return
	}
	userRes := svc.getPermissionRes(*result.Data)
	userRes.Roles = api.GetBaseRes(result.Data.Roles)
	c.JSON(http.StatusOK, res.SuccessResponse(userRes))
}
