package product

import (
	"github.com/gin-gonic/gin"
	"net/http"
	"sghgogs.com/micro/shopping-service-client/api"
	model_res "sghgogs.com/micro/shopping-service-client/domain/model/response"
	res "sghgogs.com/micro/shopping-service-client/response"
	"sghgogs.com/micro/shopping-service-client/utils"
	pb "sghgogs.com/micro/shopping-service-service/proto"
	"strconv"
)

type ListResponse struct {
	Items      []model_res.Product `json:"items"`
	TotalCount int64               `json:"total_count"`
}

// GetProductList
// @summary 列表
// @Tags 商品管理
// @Accept json
// @Produce json
// @Param Authorization header string true "Bearer 用户令牌"
// @Param page query int true "页码"
// @Param page_size query int true "每页数量"
// @Param keyword query string false "关键字"
// @Success 200 {object} response.ApiResponse{data=ListResponse} "成功"
// @Failure 400 {object} response.ApiResponse "请求错误"
// @Failure 500 {object} response.ApiResponse "内部错误"
// @Router /admin/v1/api/products [get]
func (svc *Product) GetProductList(c *gin.Context) {
	keyword := c.Query("keyword")
	queryPage := c.DefaultQuery("page", "1")
	queryPageSize := c.DefaultQuery("page_size", "10")
	page, _ := strconv.Atoi(queryPage)
	pageSize, _ := strconv.Atoi(queryPageSize)
	ctx, _ := utils.CreateContextWithToken(c, svc.Namespace, "ShoppingProductService.GetProductList")
	list, err := svc.Service.GetProductList(ctx, &pb.GetProductListRequest{
		Page:     int32(page),
		PageSize: int32(pageSize),
		Keyword:  keyword,
	})
	if err != nil {
		code, mgs := res.MicroErrorRequest(err)
		c.JSON(code, res.ErrorResponse(code, mgs))
		return
	}
	c.JSON(http.StatusOK, res.SuccessResponse(ListResponse{
		TotalCount: list.TotalCount,
		Items:      api.GetProductsRes(list.Items),
	}))
}

// GetProduct
// @summary 详情
// @Tags 商品管理
// @Accept json
// @Produce json
// @Param Authorization header string true "Bearer 用户令牌"
// @Param    id   path      int  true  "product id"
// @Success 200 {object} response.ApiResponse{data=model_res.Product} "成功"
// @Failure 400 {object} response.ApiResponse "请求错误"
// @Failure 500 {object} response.ApiResponse "内部错误"
// @Router /admin/v1/api/product/{product_id} [get]
func (svc *Product) GetProduct(c *gin.Context) {
	productID, err := strconv.ParseInt(c.Param("product_id"), 10, 64)
	if err != nil || productID <= 0 {
		c.JSON(http.StatusBadRequest, gin.H{"error": "Invalid productID"})
		return
	}
	ctx, _ := utils.CreateContextWithToken(c, svc.Namespace, "ShoppingProductService.GetProduct")
	result, err := svc.Service.GetProduct(ctx, &pb.GetProductRequest{ProductId: productID})
	if err != nil {
		code, mgs := res.MicroErrorRequest(err)
		c.JSON(http.StatusBadRequest, res.ErrorResponse(code, mgs))
		return
	}
	item := api.GetProductResItem(result.Data)
	item.Category = api.GetCategory(result.Data.Category)
	c.JSON(http.StatusOK, res.SuccessResponse(item))
}
