package product

import (
	"github.com/gin-gonic/gin"
	"net/http"
	res "sghgogs.com/micro/shopping-service-client/response"
	"sghgogs.com/micro/shopping-service-client/utils"
	pb "sghgogs.com/micro/shopping-service-service/proto"
	"strconv"
)

type UpdateProductRequest struct {
	Name          string `json:"name"  binding:"required"`          // 商品名称
	Description   string `json:"description"`                       // 描述
	Price         int32  `json:"price" binding:"required"`          // 商品价格
	StockQuantity int32  `json:"stock_quantity" binding:"required"` // 库存数量
	CategoryId    int64  `json:"category_id" binding:"required"`    // 分类ID
	ImageUrl      string `json:"image_url" binding:"required"`      // 商品图片
}

// UpdateProduct
// @summary 更新
// @Tags 商品管理
// @Accept json
// @Produce json
// @Param Authorization header string true "Bearer 用户令牌"
// @Param    id   path      int  true  "角色 ID"
// @Param body body UpdateProductRequest true "请求body"
// @Success 200 {object} response.ApiResponse "成功"
// @Failure 400 {object} response.ApiResponse "请求错误"
// @Failure 500 {object} response.ApiResponse "内部错误"
// @Router /admin/v1/api/product/{product_id} [put]
func (svc *Product) UpdateProduct(c *gin.Context) {
	var req UpdateProductRequest
	productID, err := strconv.ParseInt(c.Param("product_id"), 10, 64)
	if err != nil || productID <= 0 {
		c.JSON(http.StatusBadRequest, res.ErrorResponse(http.StatusBadRequest, "Invalid productID"))
		return
	}
	if err = c.BindJSON(&req); err != nil {
		c.JSON(http.StatusBadRequest, res.ErrorResponse(http.StatusBadRequest, err.Error()))
		return
	}

	ctx, _ := utils.CreateContextWithToken(c, svc.Namespace, "ShoppingProductService.UpdateProduct")
	_, err = svc.Service.UpdateProduct(ctx, &pb.UpdateProductRequest{
		ProductId:     productID,
		Name:          req.Name,
		Description:   req.Description,
		Price:         req.Price,
		StockQuantity: req.StockQuantity,
		CategoryId:    req.CategoryId,
		ImageUrl:      req.ImageUrl,
	})
	if err != nil {
		code, mgs := res.MicroErrorRequest(err)
		c.JSON(code, res.ErrorResponse(code, mgs))
		return
	}
	c.JSON(http.StatusOK, res.SuccessResponse(nil))
}
