package role

import (
	"github.com/gin-gonic/gin"
	"net/http"
	"sghgogs.com/micro/common"
	res "sghgogs.com/micro/shopping-service-client/response"
	"sghgogs.com/micro/shopping-service-client/utils"
	pb "sghgogs.com/micro/shopping-service-service/proto"
)

type CreateRoleRequest struct {
	Name        string  `json:"name" binding:"required"` // 用户名
	Description string  `json:"description"`             // 描述
	Users       []int64 `json:"users"`                   // 管理员
	Permissions []int64 `json:"permissions"`             // 权限项
}

// CreateRole
// @summary 新增角色
// @Tags 角色管理 Roles
// @Accept json
// @Produce json
// @Param Authorization header string true "Bearer 用户令牌"
// @Param body body CreateRoleRequest true "请求body"
// @Success 200 {object} response.ApiResponse "成功"
// @Failure 400 {object} response.ApiResponse "请求错误"
// @Failure 500 {object} response.ApiResponse "内部错误"
// @Router /admin/v1/api/role [post]
func (svc *Role) CreateRole(c *gin.Context) {
	var req CreateRoleRequest
	if err := c.ShouldBindJSON(&req); err != nil {
		c.JSON(http.StatusBadRequest, res.ErrorResponse(http.StatusBadRequest, err.Error()))
		return
	}
	if isBool, err := common.ValidateNumericInt64(req.Users); !isBool {
		c.JSON(http.StatusBadRequest, res.ErrorResponse(http.StatusBadRequest, err.Error()))
		return
	}
	if isBool, err := common.ValidateNumericInt64(req.Permissions); !isBool {
		c.JSON(http.StatusBadRequest, res.ErrorResponse(http.StatusBadRequest, err.Error()))
		return
	}
	ctx, _ := utils.CreateContextWithToken(c, svc.Namespace, "ShoppingRoleService.CreateRole")
	_, err := svc.Service.CreateRole(ctx, &pb.CreateRoleRequest{
		Name:        req.Name,
		Description: req.Description,
		Users:       req.Users,
		Permissions: req.Permissions,
	})
	if err != nil {
		code, mgs := res.MicroErrorRequest(err)
		c.JSON(code, res.ErrorResponse(code, mgs))
		return
	}
	c.JSON(http.StatusOK, res.SuccessResponse(nil))
}
