package role

import (
	"github.com/gin-gonic/gin"
	"net/http"
	"sghgogs.com/micro/shopping-service-client/api"
	"sghgogs.com/micro/shopping-service-client/domain/model/base"
	model_res "sghgogs.com/micro/shopping-service-client/domain/model/response"
	res "sghgogs.com/micro/shopping-service-client/response"
	"sghgogs.com/micro/shopping-service-client/utils"
	pb "sghgogs.com/micro/shopping-service-service/proto"
	"strconv"
)

type ListResponse struct {
	Items      []model_res.Role `json:"items"`
	TotalCount int64            `json:"total_count"`
}

// GetRoleList
// @summary 列表
// @Tags 角色管理 Roles
// @Accept json
// @Produce json
// @Param Authorization header string true "Bearer 用户令牌"
// @Param page query int true "页码"
// @Param page_size query int true "每页数量"
// @Param keyword query string false "关键字"
// @Param status query string false "状态" Enums(enabled, disabled)
// @Success 200 {object} response.ApiResponse{data=ListResponse} "成功"
// @Failure 400 {object} response.ApiResponse "请求错误"
// @Failure 500 {object} response.ApiResponse "内部错误"
// @Router /admin/v1/api/roles [get]
func (svc *Role) GetRoleList(c *gin.Context) {
	keyword := c.Query("keyword")
	queryPage := c.DefaultQuery("page", "1")
	queryPageSize := c.DefaultQuery("page_size", "10")
	page, _ := strconv.Atoi(queryPage)
	pageSize, _ := strconv.Atoi(queryPageSize)

	ctx, _ := utils.CreateContextWithToken(c, svc.Namespace, "ShoppingRoleService.GetRoleList")
	list, err := svc.Service.GetRoleList(ctx, &pb.GetRoleListRequest{
		Page:     int32(page),
		PageSize: int32(pageSize),
		Keyword:  keyword,
	})
	if err != nil {
		code, mgs := res.MicroErrorRequest(err)
		c.JSON(code, res.ErrorResponse(code, mgs))
		return
	}
	users := make([]model_res.Role, 0)
	for _, item := range list.Items {
		users = append(users, api.GetRoleRes(item))
	}
	c.JSON(http.StatusOK, res.SuccessResponse(ListResponse{
		TotalCount: list.TotalCount,
		Items:      users,
	}))
}

type AllResponse struct {
	Items []base.ListMapItem `json:"items"`
}

// GetAllRoles
// @summary 列表
// @Tags 角色管理 Roles
// @Accept json
// @Produce json
// @Param Authorization header string true "Bearer 用户令牌"
// @Success 200 {object} response.ApiResponse{data=AllResponse} "成功"
// @Failure 400 {object} response.ApiResponse "请求错误"
// @Failure 500 {object} response.ApiResponse "内部错误"
// @Router /admin/v1/api/all_roles [get]
func (svc *Role) GetAllRoles(c *gin.Context) {
	ctx, _ := utils.CreateContextWithToken(c, svc.Namespace, "ShoppingRoleService.GetAllRoles")
	list, err := svc.Service.GetAllRoles(ctx, &pb.GetAllRolesRequest{})
	if err != nil {
		code, mgs := res.MicroErrorRequest(err)
		c.JSON(code, res.ErrorResponse(code, mgs))
		return
	}
	c.JSON(http.StatusOK, res.SuccessResponse(AllResponse{
		Items: api.GetBaseRes(list.Items),
	}))
}

// GetRole
// @summary 详情
// @Tags 角色管理 Roles
// @Accept json
// @Produce json
// @Param Authorization header string true "Bearer 用户令牌"
// @Param    id   path      int  true  "角色ID"
// @Success 200 {object} response.ApiResponse{data=model_res.Role} "成功"
// @Failure 400 {object} response.ApiResponse "请求错误"
// @Failure 500 {object} response.ApiResponse "内部错误"
// @Router /admin/v1/api/role/{role_id} [get]
func (svc *Role) GetRole(c *gin.Context) {
	roleID, err := strconv.ParseInt(c.Param("role_id"), 10, 64)
	if err != nil || roleID <= 0 {
		c.JSON(http.StatusBadRequest, gin.H{"error": "Invalid roleID"})
		return
	}
	ctx, _ := utils.CreateContextWithToken(c, svc.Namespace, "ShoppingRoleService.GetRole")
	result, err := svc.Service.GetRole(ctx, &pb.GetRoleRequest{RoleId: roleID})
	if err != nil {
		code, mgs := res.MicroErrorRequest(err)
		c.JSON(http.StatusBadRequest, res.ErrorResponse(code, mgs))
		return
	}
	userRes := api.GetRoleRes(result.Data)
	userRes.Users = api.GetBaseRes(result.Data.Users)
	userRes.Permissions = api.GetBaseRes(result.Data.Permissions)
	c.JSON(http.StatusOK, res.SuccessResponse(userRes))
}
