package role

import (
	"github.com/gin-gonic/gin"
	"net/http"
	"sghgogs.com/micro/common"
	res "sghgogs.com/micro/shopping-service-client/response"
	"sghgogs.com/micro/shopping-service-client/utils"
	pb "sghgogs.com/micro/shopping-service-service/proto"
	"strconv"
)

type UpdateRoleRequest struct {
	Description string  `json:"description"` // 描述
	Users       []int64 `json:"users"`       // 管理员
	Permissions []int64 `json:"permissions"` // 权限项
}

// UpdateRole
// @summary 更新
// @Tags 角色管理 Roles
// @Accept json
// @Produce json
// @Param Authorization header string true "Bearer 用户令牌"
// @Param    id   path      int  true  "角色 ID"
// @Param body body UpdateRoleRequest true "请求body"
// @Success 200 {object} response.ApiResponse "成功"
// @Failure 400 {object} response.ApiResponse "请求错误"
// @Failure 500 {object} response.ApiResponse "内部错误"
// @Router /admin/v1/api/role/{role_id} [put]
func (svc *Role) UpdateRole(c *gin.Context) {
	var req UpdateRoleRequest
	roleID, err := strconv.ParseInt(c.Param("role_id"), 10, 64)
	if err != nil || roleID <= 0 {
		c.JSON(http.StatusBadRequest, res.ErrorResponse(http.StatusBadRequest, common.ErrorMessage[common.InvalidUserID]))
		return
	}
	if err = c.BindJSON(&req); err != nil {
		c.JSON(http.StatusBadRequest, res.ErrorResponse(http.StatusBadRequest, err.Error()))
		return
	}
	if len(req.Permissions) > 0 {
		if isBool, err := common.ValidateNumericInt64(req.Permissions); !isBool {
			c.JSON(http.StatusBadRequest, res.ErrorResponse(http.StatusBadRequest, err.Error()))
			return
		}
	}
	if len(req.Users) > 0 {
		if isBool, err := common.ValidateNumericInt64(req.Users); !isBool {
			c.JSON(http.StatusBadRequest, res.ErrorResponse(http.StatusBadRequest, err.Error()))
			return
		}
	}
	ctx, _ := utils.CreateContextWithToken(c, svc.Namespace, "ShoppingRoleService.UpdateRole")
	_, err = svc.Service.UpdateRole(ctx, &pb.UpdateRoleRequest{
		RoleId:      roleID,
		Description: req.Description,
		Users:       req.Users,
		Permissions: req.Permissions,
	})
	if err != nil {
		code, mgs := res.MicroErrorRequest(err)
		c.JSON(code, res.ErrorResponse(code, mgs))
		return
	}
	c.JSON(http.StatusOK, res.SuccessResponse(nil))
}
