package router

import (
	"github.com/gin-gonic/gin"
	"github.com/pkg/errors"
	"github.com/sirupsen/logrus"
	swaggerFiles "github.com/swaggo/files"
	ginSwagger "github.com/swaggo/gin-swagger"
	"go-micro.dev/v4/client"
	"net/http"
	auth_pb "sghgogs.com/micro/auth-service/proto"
	"sghgogs.com/micro/common"
	_ "sghgogs.com/micro/shopping-service-client/docs"
	admin_adminuser "sghgogs.com/micro/shopping-service-client/router/adminuser"
	admin_auth "sghgogs.com/micro/shopping-service-client/router/auth"
	admin_categor "sghgogs.com/micro/shopping-service-client/router/categor"
	"sghgogs.com/micro/shopping-service-client/router/order"
	admin_permission "sghgogs.com/micro/shopping-service-client/router/permission"
	admin_product "sghgogs.com/micro/shopping-service-client/router/product"
	admin_role "sghgogs.com/micro/shopping-service-client/router/role"
	"sghgogs.com/micro/shopping-service-client/router/upload"
	admin_user "sghgogs.com/micro/shopping-service-client/router/user"
	"sghgogs.com/micro/shopping-service-client/utils"
	"sghgogs.com/micro/shopping-service-client/utils/authutil"
	"sghgogs.com/micro/shopping-service-client/utils/baseservice"
	"sghgogs.com/micro/shopping-service-client/utils/miniohelper"
	"sghgogs.com/micro/shopping-service-client/utils/tracing"
	"strconv"
	"time"
)

func Logger() gin.HandlerFunc {
	// 创建一个新的 Logrus 日志记录器
	log := logrus.New()
	// 设置日志级别
	// log.SetLevel(logrus.DebugLevel)
	log.SetLevel(logrus.TraceLevel)
	return func(c *gin.Context) {
		start := time.Now()
		c.Next()
		latency := time.Since(start)
		log.Printf("[%s] %s %s %s %s",
			c.Request.Method,
			c.Request.RequestURI,
			c.ClientIP(),
			latency,
			c.Request.UserAgent(),
		)
	}
}

func ExpirationFromMiddleware(namespace string, client client.Client) gin.HandlerFunc {
	return func(c *gin.Context) {
		tokenData, _ := authutil.JWTAuthService.GetTeamFromRedis(c, namespace)
		if tokenData == nil {
			api := auth_pb.NewTeamService("authorizationservice", client)
			ctx, _ := utils.CreateContextWithToken(c, "authorizationservice", "TeamService.GetTeam")
			team, err := api.GetTeam(ctx, &auth_pb.GetTeamRequest{
				Keyword: namespace,
			})
			if err != nil {
				c.JSON(http.StatusUnauthorized, gin.H{
					"code":    http.StatusUnauthorized,
					"message": errors.New(common.ErrorMessage[common.InternalServerErrorErrorCode]),
				})
				c.Abort() // 终止请求处理
				return
			}
			// ErrTeamTokenExpired
			inspect, _ := authutil.JWTAuthService.Inspect(team.Data.Credential)
			if value, ok := inspect.Metadata["expiry"]; ok {
				expiry, _ := strconv.ParseInt(value, 10, 64)
				// 已过期
				if !time.Now().Before(time.Unix(expiry, 0)) {
					c.JSON(http.StatusUnauthorized, gin.H{
						"code":    http.StatusUnauthorized,
						"message": errors.New(common.ErrorMessage[common.ErrTeamTokenExpired]),
					})
					c.Abort() // 终止请求处理
					return
				}
				if err = authutil.JWTAuthService.SetToTeamRedis(c, namespace, &authutil.TeamTokenData{
					Token:          team.Data.Credential,
					ExpirationTime: expiry,
				}); err != nil {
					c.JSON(http.StatusUnauthorized, gin.H{
						"code":    http.StatusUnauthorized,
						"message": errors.New(common.ErrorMessage[common.FailedRedisInternalServerErrorCode]),
					})
					c.Abort() // 终止请求处理
					return
				}
				c.Next()
			} else {
				c.JSON(http.StatusUnauthorized, gin.H{
					"code":    http.StatusUnauthorized,
					"message": errors.New(common.ErrorMessage[common.ErrTeamTokenExpired]),
				})
				c.Abort() // 终止请求处理
				return
			}
		} else {
			c.Next()
		}
	}

}

func NewRouter(namespace string, client client.Client, minio *miniohelper.MinioClient) *gin.Engine {
	r := gin.Default()
	r.Use(Logger())
	r.Use(tracing.NewTracingContextWrapper())
	// 发布的时候要开启
	// todo
	// r.Use(ExpirationFromMiddleware(namespace, client))
	// 发布的时候要开启
	adminAuthRouter := admin_auth.NewAuthRouter(namespace, client)
	adminUserAdminRouter := admin_adminuser.NewAdminUserRouter(namespace, client)
	adminCategoryRouter := admin_categor.NewCategoryRouter(namespace, client)
	adminRoleRouter := admin_role.NewRoleRouter(namespace, client)
	adminUserRouter := admin_user.NewUserRouter(namespace, client)
	adminPermissionRouter := admin_permission.NewPermissionRouter(namespace, client)
	updateRouter := upload.NewUploadRouter(namespace, client, *minio)
	adminProductRouter := admin_product.NewProductRouter(namespace, client)
	orderRouter := order.NewOrderRouter(namespace, client)
	baseservice.RegisterRouters(r, adminAuthRouter, adminUserAdminRouter, adminCategoryRouter, adminRoleRouter, adminPermissionRouter, adminUserRouter, updateRouter, adminProductRouter, orderRouter)
	r.GET("/_healthz", func(c *gin.Context) {
		c.String(http.StatusOK, "ok")
	})
	r.GET("/swagger/*any", ginSwagger.WrapHandler(swaggerFiles.Handler))
	return r
}
