package miniohelper

import (
	"fmt"
	"github.com/minio/minio-go"
	"log"
	"os"
	"path/filepath"
	"strings"
	"time"
)

type MinioClient struct {
	Client *minio.Client
}

// NewMinioClient 初始化minio
func NewMinioClient(endpoint, accessKey, secretKey string) (*MinioClient, error) {
	// 初始化 Minio 客户端
	minioClient, err := minio.New(endpoint, accessKey, secretKey, false)
	if err != nil {
		log.Println("new minio client fail: ", err)
		return nil, err
	}
	return &MinioClient{
		Client: minioClient,
	}, nil
}
func generateDynamicFilename(filePath string) string {
	// 使用 filepath.Base 获取文件名
	fileName := filepath.Base(filePath)
	// 使用 strings.LastIndex 获取最后一个点的索引
	lastDotIndex := strings.LastIndex(fileName, ".")
	// 如果文件名中包含点，则截取后缀，否则使用原始文件名
	var prefix, suffix string
	if lastDotIndex != -1 {
		prefix = fileName[:lastDotIndex]
		suffix = fileName[lastDotIndex:]
	} else {
		prefix = fileName
		suffix = ""
	}
	// 使用时间戳作为文件名的前缀
	timestamp := time.Now().Format("20060102150405") // 格式化为yyyyMMddHHmmss
	return timestamp + "_" + prefix + suffix
}

// UploadFile 上传文件
func (m *MinioClient) UploadFile(bucketName, name, objectName, filePath string) (string, error) {
	// 打开本地文件
	file, err := os.Open(filePath)
	if err != nil {
		log.Printf("open filePath: %s fail: %s", filePath, err)
		return "", err
	}
	defer file.Close()

	// Get file stats
	fileInfo, err := file.Stat()
	if err != nil {
		return "", err
	}
	filename := fmt.Sprintf("%v/%v", name, generateDynamicFilename(objectName))
	// 上传文件到存储桶
	_, err = m.Client.PutObject(bucketName, filename, file, fileInfo.Size(), minio.PutObjectOptions{})
	if err != nil {
		log.Println("putObject fail: ", err)
		return "", err
	}

	fmt.Println("Successfully uploaded", filename)

	return filename, nil
}

// DownloadFile 下载文件
func (m *MinioClient) DownloadFile(bucketName, objectName, filePath string) error {
	// 创建本地文件
	file, err := os.Create(filePath)
	if err != nil {
		return err
	}
	defer file.Close()

	// 下载存储桶中的文件到本地
	err = m.Client.FGetObject(bucketName, objectName, filePath, minio.GetObjectOptions{})
	if err != nil {
		return err
	}

	fmt.Println("Successfully downloaded", objectName)
	return nil
}

// DeleteFile 删除文件
func (m *MinioClient) DeleteFile(bucketName, objectName string) (bool, error) {
	// 删除存储桶中的文件
	err := m.Client.RemoveObject(bucketName, objectName)
	if err != nil {
		log.Println("remove object fail: ", err)
		return false, err
	}

	fmt.Println("Successfully deleted", objectName)
	return true, err
}

// ListObjects 列出文件
func (m *MinioClient) ListObjects(bucketName, prefix string) ([]string, error) {
	var objectNames []string

	for object := range m.Client.ListObjects(bucketName, prefix, true, nil) {
		if object.Err != nil {
			return nil, object.Err
		}

		objectNames = append(objectNames, object.Key)
	}

	return objectNames, nil
}

// GetPresignedGetObject 返回对象的url地址，有效期时间为expires
func (m *MinioClient) GetPresignedGetObject(bucketName string, objectName string, expires time.Duration) (string, error) {
	object, err := m.Client.PresignedGetObject(bucketName, objectName, expires, nil)
	if err != nil {
		log.Println("get object fail: ", err)
		return "", err
	}

	return object.String(), nil
}
